#include <iostream>
#include <boost/timer_new.hpp>

using namespace std;

// Uncomment those at your needs
//#define TEST_SECOND_TIMER
//#define TEST_WIN32_TIMERS

inline void mysleep(int milliseconds)
{
	cout << "Sleeping for " << (milliseconds / 1000.0) << " seconds" << endl;
	#ifdef BOOST_WINDOWS
	Sleep(milliseconds);
	#else
	usleep(milliseconds * 1000);
	#endif
}

template <class timer>
inline void print_value(const timer& tmr)
{
	cout << "Current value: " << tmr.elapsed().seconds() << "." << tmr.elapsed().fractional_seconds()  << endl;
}

template<class timer>
inline void run_test()
{
	cout << "Starting timer" << endl;
	timer tmr;
	mysleep(200);
	print_value(tmr);
	mysleep(200);
	print_value(tmr);

	cout << "Stopping timer" << endl;
	tmr.pause();
	print_value(tmr);

	mysleep(1000);
	cout << "This last sleep should not be timed as the timer is stopped" << endl;
	print_value(tmr);

	cout << "Resuming timer" << endl;
	tmr.resume();
	mysleep(200);
	print_value(tmr);
	mysleep(200);
	print_value(tmr);

	cout << "Restarting timer" << endl;
	tmr.reset();
	mysleep(200);
	print_value(tmr);
	mysleep(1500);
	print_value(tmr);
	mysleep(200);
	print_value(tmr);
}

int main()
{
	cout << "[Testing with microsec timer]" << endl;
	run_test<boost::microsec_timer>();
	cout << "------------------------" << endl;

	#ifdef TEST_SECOND_TIMER
	cout << "[Testing with second timer]" << endl;
	run_test<boost::second_timer>();
	cout << "------------------------" << endl;
	#endif

	#if defined(BOOST_WINDOWS) && defined(TEST_WIN32_TIMERS)
	cout << "[Testing with QPC timer]" << endl;
	run_test<boost::qpc_timer>();
	cout << "------------------------" << endl;
	#endif

	#if defined(BOOST_WINDOWS) && defined(TEST_WIN32_TIMERS)
	cout << "[Testing with timeGetTime timer]" << endl;
	run_test<boost::tgt_timer>();
	cout << "------------------------" << endl;
	#endif

	return 0;
}
