#ifndef BOOST_TGT_TIMER_HPP____
#define BOOST_TGT_TIMER_HPP____

// (C) Copyright Vaucher Philippe 2006
// Use, modification and distribution is subject to the 
// Boost Software License, Version 1.0. (See accompanying
// file LICENSE-1.0 or http://www.boost.org/LICENSE-1.0)


// -------------------------- Includes --------------------------

// Boost headers
#include <boost/date_time/posix_time/posix_time.hpp>


// -------------------------- Code --------------------------

namespace boost
{

	#ifndef BOOST_WINDOWS
		#error "timeGetTime() based timer is only available on windows"
	#endif

	#pragma comment(lib, "winmm.lib")

	#define WIN32_LEAN_AND_MEAN
	#include <windows.h>

	class tgt_timer
	{

	public:

		typedef boost::posix_time::ptime time_type;
		typedef time_type::time_duration_type time_duration_type;

		enum start_options { AUTO_START, MANUAL_START };

		tgt_timer(time_duration_type initial_duration = time_duration_type(0, 0, 0), start_options start_op = AUTO_START) : m_start(0), m_elapsed(initial_duration)
		{
			if(start_op == AUTO_START)
				start();
		}

		void start() 
		{
			if(!m_start)
				m_start = timeGetTime();
		}

		void restart()
		{
			reset();
			start();
		}

		time_duration_type elapsed() const
		{
			if(m_start)
			{
				boost::uint32_t current = timeGetTime();
				boost::uint64_t milliseconds = (current - m_start);
				boost::uint32_t seconds = milliseconds / 1000;
				boost::uint32_t minutes = seconds / 60;
				boost::uint32_t hours = minutes / 60;
				milliseconds = milliseconds % 1000;
				m_elapsed += time_duration_type(hours, minutes, seconds, milliseconds);
				m_start = current;
			}
			return m_elapsed;
		}

		void pause()
		{
			boost::uint32_t current = timeGetTime();
			boost::uint64_t milliseconds = (current - m_start);
			boost::uint32_t seconds = milliseconds / 1000;
			boost::uint32_t minutes = seconds / 60;
			boost::uint32_t hours = minutes / 60;
			milliseconds = milliseconds % 1000;
			m_elapsed += time_duration_type(hours, minutes, seconds, milliseconds);
			m_start = 0; 
		}

		void resume()
		{
			if(!m_start)
				m_start = timeGetTime();
		}

		void reset()
		{
			m_elapsed = time_duration_type(0,0,0);
			if(m_start)
				m_start = timeGetTime();
		}


	private:

		mutable boost::uint32_t m_start;
		mutable time_duration_type m_elapsed;

	};

}

#endif
